"""The enumeration of the different supported species."""

from enum import StrEnum


class Species(StrEnum):
    """An enumeration over the different species."""

    HUMAN = "human"
    HORSE = "horse"
